//////////////////////////////////////////////////////////////////////////////////
// Module    : sx16core.h                                                       //
// Function  : Include for Sensoray 516/616 application and driver projects.    //
// Author    : Jim Lamberson, Charlie Liu                                       //
// Revision  :                                                                  //
//             May,  2005   Charlie X. Liu   Added SX16CORE_encoder_out_ctrl()  //
//                                           for video output shut off          //
//             Sept, 2004   Phillip W.       Made lin2.6 and Windows compatible //
//             May,  2004   Alexander K.     Added RTP and some playback        //
//                                           sync'ing stuff                     //
//             Oct., 2003   Charlie X. Liu   Added caption-related definitions	//
//             Sept, 2003   Jim Lamberson    release sx16-0.7                   //
//                                                                              //
// Copyright : (C) Sensoray 2003                                                //
//////////////////////////////////////////////////////////////////////////////////

#ifndef DEF_SX16CORE_H
#define DEF_SX16CORE_H

//#define TEST_PCI_616 // for test only

#include "vidcap_link.h"

#define SYS_XACTIVE_MAX_NTSC    704             //max active hor pixels, NTSC;
#define SYS_XACTIVE_MAX_PAL     704             //....................., PAL;
#define SYS_YACTIVE_MAX_NTSC    480             //max vert active pixels, NTSC;
#define SYS_YACTIVE_MAX_PAL     576             //......................, PAL;


// Types --------------------------------------------------------------------
typedef void *  HBD;	// Logical handle to X16 board.
///////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////// STRUCTURAL CONFIGURATION CONSTANTS  ////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

// SM dma fifo transfer size.  Theoretically, this may be any power of 2 in the range 8 to 32768.  The SM documentation suggests
// that this be specified as an integral multiple of 256.  In practice, however, 31K is the only value that seems to work reliably 
// for both the Play and Record modes.
#define DFLT_DMA_SIZE		( 1024 * 31 )	// SM's mpeg dma transfer size.

// Block transfer size, in bytes, between the SM and our private mpeg buffer.  Larger sizes work in playback mode,
// but will cause problems in record mode.
#define CHUNK_SIZE			( 1024 )		// SM's block transfer size in bytes.
#define CHUNK_SIZE_PLAY		( 4*1024 )// SM's block transfer size in bytes.
#ifdef  SX16_KERNEL
#ifdef SX16_BUILD_SRAY
#include "rtp.h"
#endif
#endif 


// Default A/V configuration settings.
#define DFLT_A_BITRATE		11					// Enumerated audio bit rate = 224 kbps (assumes audio format = MPEG-1 layer 2).
#define DFLT_AFORMAT		AFMT_MPEG1_LAYER2	// Audio format = MPEG-1 layer 2.
#define DFLT_ASR			1					// Enumerated audio sample rate = 48 KHz.
#define DFLT_AV_SELECT		AVSEL_BOTH			// Mux A/V select = both audio and video.
#define DFLT_BASS			0					// Audio audio_bass level.
#define DFLT_BRIGHTNESS		128					// Luminance brightness = nominal.
#define DFLT_CHAN			0					// Video input channel = 0, which is present on all models.
#define DFLT_COLORBARS		FALSE				// Colorbar generator disabled.
#define DFLT_CONTRAST		68					// Luminance contrast = slightly above nominal (64).
#define DFLT_FILTER			FALSE				// Video input filter disabled.
#define DFLT_GOP			15					// 
#define DFLT_INPUT_CHAN		0					// Input channel = first channel (present on both 516 and 616).
#define DFLT_INPUT_TYPE		CVIDEO				// Input type = composite video.
#define DFLT_IP				1					// I-P distance = 1 // For pAL 6/27/2007.
#define DFLT_MUTE			FALSE				// Audio muting disabled.
#define DFLT_PCM			AUDIO_PCM_RES_16	// Enumerated PCM resolution = 16 bits.
#define DFLT_PICSIZE		SIZE704_2			// Enumerated picture size = MPEG2/D1-704.
#define DFLT_PROG_NUM		0					// Program number = 0 ( Sasha's guess is 0x010 -- ??? ).
#define DFLT_READY_THRESH	4096				// Client's default "ready" threshold for our mpeg buffer.
#define DFLT_SATURATION		64					// Chrominance saturation = nominal.
#define DFLT_STEREO_MODE	STEREOMODE_STEREO	// MPEG stereo mode = stereo.
#define DFLT_STREAMTYPE		STREAMTYPE_MUX		// Stream type = Multiplexed.
#define DFLT_SYS_FORMAT		SYSFMT_OTHER		// System format = OTHER (don't change--used for all modes).
#define DFLT_TREBLE			0					// Audio audio_treble level.
#define DFLT_V_BITRATE		6000000				// Bit rate = 6 Mbps.
#define DFLT_VBR_MODE		FALSE				// Variable bit rate = disabled.
#define DFLT_VOLUME			0					// Audio audio_volume level.
#define DFLT_VSYSTEM		SYS_NTSC			 // Video system = NTSC.

// default mic settings
#define DFLT_AUDIO_INAMPGAIN  6  // 0 dB 
#define DFLT_AUDIO_AGC_ONOFF  0  //AGC off
#define DFLT_AUDIO_AGC_SENS	  1  // 0db

// Default caption settings.

#define DFLT_CAPTN_WINDOW	1					// Caption window # = 1
#define DFLT_CAPTN_XSTART	0					// Caption X start = 0
#define DFLT_CAPTN_XSTOP	0					// Caption X stop  = 0
#define DFLT_CAPTN_YSTART	0					// Caption Y start = 0
#define DFLT_CAPTN_YSTOP	0					// Caption Y stop  = 0
#define DFLT_CAPTN_SCALE	OVL_SCALE_X1		// Caption default scale = x1
#define DFLT_CAPTN_COLOR	OVL_WHITE			// Caption default color = white
#define DFLT_CAPTN_TRANSP	OVL_TRANSP_T		// Caption default transparency = transparent
/* #define DFLT_CAPTN_TEXT 	?					// Caption text = non */
#define DFLT_CAPTN_ON		CAPTION_OFF			// Caption window on/off = off
// Dependent on other settings ...
#define DFLT_MUX_FORMAT		MUX_FORMAT_DVD		// Mux format = DVD. VCD=mpeg1, DVD=mpeg2, VES=ves, AES=aes.  Depends on STREAMTYPE and PICSIZE.


// ioctl constants ----------------------------------------------------

// Video system.
#define SYS_PAL						1			// PAL.
#define SYS_NTSC					2			// NTSC.

// Video input type.
#define CVIDEO						0			// Composite video.
#define SVIDEO						1			// S-Video.

// Video input status bits.
#define VIDEO_DET					1			// Video signal detected.
#define F60HZ_DET					0x20		// Field frequency 60 Hz detected.

// Picture sizes/stream format.
#define SIZE176_1					4			// QSIF,     MPEG-1, NTSC-176x112 PAL-176x144
#define SIZE352_1					3			// SIF,      MPEG-1, NTSC-352x240 PAL-352x288
#define SIZE352_2					2			// 1/2 D1,   MPEG-2, NTSC-352x480 PAL-352x576
#define SIZE480_2					1			// 2/3 D1,   MPEG-2, NTSC-480x480 PAL-480x576 SVCD mux(standard)
#define SIZE640_2					6			// D1 H:640, MPEG-2, NTSC-640x480 PAL-640x576 Like mode 5, but clipped to 640
#define SIZE704_2					0			// D1 H:704, MPEG-2, NTSC-704x480 PAL-704x576
#define SIZE720_2					5			// D1 H:720, MPEG-2, NTSC-720x480 PAL-720x576
#define SIZE352_1_2					7			// SIF,      MPEG-1, NTSC-352x240 PAL-352x288, MPEG1 stream with MPEG2 mux
#define SIZE480_2_2					8			// 2/3 D1,   MPEG-2, NTSC-480x480 PAL-480x576, DVD MUX(for playback)

// Stream type.
#define STREAMTYPE_MUX				0			// Multiplexed program/system stream.
#define STREAMTYPE_VES				1			// Video elementary stream.
#define STREAMTYPE_AES				2			// Audio elementary stream.
#define STREAMTYPE_TRANSPORT		3			// Transport stream

// Audio encoding format.
#define AFMT_MPEG1_LAYER1			0			// MPEG-1 layer 1.
#define AFMT_MPEG1_LAYER2			1			// MPEG-1 layer 2.

#define AFMT_MPEG1_LAYER3                       2 
///////////////////////////// LICENSE REQUIRED //////////////////////////////////
//////// Please contact Dolby Laboratories, Inc. before use this format /////////
// Note: some editions of SM microcode do not support this format.
#define AFMT_AC3					3			// AC3 2-channel.
/////////////////////////////////////////////////////////////////////////////////

// Stereo mode for MPEG audio.
#define STEREOMODE_STEREO			0			// Stereo.
#define STEREOMODE_JOINT			1			// Joint stereo (intensity or ms).
#define STEREOMODE_DUAL				2			// Dual channel.
#define STEREOMODE_SINGLE			3			// Single channel.

// Audio sampling rate.
#define AUDIO_SAMPLERATE_44KHZ		0			// 44.1 KHz.
#define AUDIO_SAMPLERATE_48KHZ		1			// 48 KHz.
#define AUDIO_SAMPLERATE_32KHZ		2			// 32 KHz.

// Audio PCM resolution.
#define AUDIO_PCM_RES_16			0			// 16 bits.
#define AUDIO_PCM_RES_18			1			// 18 bits.
#define AUDIO_PCM_RES_20			2			// 20 bits.
#define AUDIO_PCM_RES_24			3			// 24 bits.

// Multiplexing audio/video select.  Not applicable to elementary streams.
#define AVSEL_BOTH					0			// Audio and video.
#define AVSEL_VIDEO_ONLY			1			// Video only.
#define AVSEL_AUDIO_ONLY			2			// Audio only.

// System format.
#define SYSFMT_SVCD					0			// Super VCD.
#define SYSFMT_VCD					1			// VCD.
#define SYSFMT_OTHER				2			// Other formats.

// Multiplexing format.
#define MUX_FORMAT_SVCD				0			// Super VCD.
#define MUX_FORMAT_VCD				1			// VCD.
#define MUX_FORMAT_DVD				2			// DVD.
#define MUX_FORMAT_CDDA				3			// CDDA.
#define MUX_FORMAT_VES				4			// Video elementary stream.
#define MUX_FORMAT_AES				5			// Audio elementary stream.
#define MUX_FORMAT_TRANSPORT		6			// Transport stream.

// Caption position.
#define MIN_XSTART					0			// Minimum X start
#define MIN_YSTART					0			// Minimum Y start
#define MAX_XSTART					700			// Maximum X start
#define MAX_YSTART					478			// Maximum Y start
#define MAX_YSTART_NTSC				478			// Maximum Y start for NTSC
#define MAX_YSTART_PAL				574			// Maximum Y start for PAL
#define MAX_XSTOP					704			// Maximum X stop
#define MAX_YSTOP					478			// Maximum Y stop
#define MAX_YSTOP_NTSC				480			// Maximum Y stop for NTSC
#define MAX_YSTOP_PAL				576			// Maximum Y stop for PAL

// Caption overlay color.
#define OVL_WHITE					0			// white.
#define OVL_BLACK					1			// black.
#define OVL_RED						2			// red.
#define OVL_GREEN					3			// green.
#define OVL_BLUE					4			// blue.
#define OVL_YELLOW					5			// yellow.
#define OVL_CYAN					6			// cyan.
#define OVL_MAGENTA					7			// magenta.

// Caption overlay scale.
#define OVL_SCALE_X1				0			// scale = x1.
#define OVL_SCALE_X2				1			// scale = x2.
#define OVL_SCALE_X2V				2			// scale = x2 (only vertical).
#define OVL_SCALE_X4				3			// scale = x4.

// Caption transparency attribute.
#define OVL_TRANSP_T				1			// transparent.
#define OVL_TRANSP_BBKGND			2			// black background.
#define OVL_TRANSP_T_BKGND			3			// transparent text on a background.
#define OVL_TRANSP_B_BKGRD			4			// black text on a background.

// Caption on/off.
#define CAPTION_OFF					0			// caption off.
#define CAPTION_ON					1			// caption on.

// Error codes.
#define ERR_I2C 					0x0001		// I2C bus error.
#define ERR_UCODE_NOTFOUND			0x0002		// Microcode can't be opened.
#define ERR_UCODE_NOUPLOAD			0x0004		// Mpeg codec not acknowledging microcode upload.
#define ERR_MEM_ALLOC				0x0008		// Memory allocation error.
#define ERR_DEBI_TIMEOUT			0x0010		// DEBI transaction timed out (616 only).

// Mpeg codec microcode selectors.
#define MPEG_RECORD					0			// Convert stream to MPEG.
#define MPEG_PLAY					1			// Convert MPEG to stream.

// Attribute identifiers for SX16CORE_attr_get() and SX16CORE_attr_set().
#define ATTR_audio_bass				0			// Audio audio_bass level.
#define ATTR_audio_bitrate			1			// Audio bit rate.
#define ATTR_audio_format			2			// Audio format.
#define ATTR_audio_mute				3			// Audio muting.
#define ATTR_audio_pcm_resolution	4			// Audio PCM resolution.
#define ATTR_audio_sample_rate		5			// Audio sample rate.
#define ATTR_audio_stereo_mode		6			// Audio stereo mode.
#define ATTR_audio_treble			7			// Audio treble level.
#define ATTR_audio_volume			8			// Audio volume level.
// see more audio options below( added later for backward compatibility with existing drivers)
#define ATTR_av_select				9			// Audio/video select.
#define ATTR_video_bitrate			10			// Video bit rate in bps.
#define ATTR_video_brightness		11			// Video brightness level.
#define ATTR_video_chan				12			// Video input channel number.
#define ATTR_video_colorbars_enable	13			// Video color bar generation enable.
#define ATTR_video_contrast			14			// Video contrast level.
#define ATTR_video_filter_enable	15			// Video filter enable.
#define ATTR_video_m				16			// GOP length.
#define ATTR_video_n				17			// I-P distance.
#define ATTR_video_picsize			18			// Picture size and compression type (fundamental).
#define ATTR_video_saturation		19			// Video video_saturation.
#define ATTR_video_type				20			// Video input type: svideo or composite.
#define ATTR_video_system			21			// Video system.
#define ATTR_video_system_format	22			// Video system format (fundamental).
#define ATTR_video_vbr_enable		23			// Variable bit rate enable.
#define ATTR_mux_format				24			// Stream multiplex format (fundamental).
#define ATTR_program_number			25			// Program number.
#define ATTR_mpeg					26			// Picture size and compression type (derived).
#define ATTR_stream_type			27			// Stream multiplex type (derived).
// microphone audio input settings.  Leave as default for line-in
#define ATTR_audio_inampgain        28          // audio input channel amplifier 0x00(-3db) to 0x7f (+60.5dB) in steps of 0.5db
#define ATTR_audio_agc_onoff        29          // microphone AGC.  0--off, 1--on.  If on, ATTR_audio_inampgain is ignored.
#define ATTR_audio_agc_sens         30          // microphone sensitivity for AGC only. 0=-3db, 1=0db, 2=+3db, 3=+9, 4=+15, 5=+21, 6=+27db



#define NUM_ATTR_IDS				31			// Number of legal attribute identifiers.

// Attribute identifiers for SX16CORE_caption_get() and SX16CORE_caption_set().
#define CAPTN_window				0			// Caption window #.
#define CAPTN_xstart				1			// Caption x start.
#define CAPTN_xstop					2			// Caption x stop.
#define CAPTN_ystart				3			// Caption y start.
#define CAPTN_ystop					4			// Caption y stop.
#define CAPTN_scale					5			// Caption scale.
#define CAPTN_color					6			// Caption color.
#define CAPTN_transp				7			// Caption transparency.
#define CAPTN_text					8			// Caption text.
#define CAPTN_on					9			// Caption on/off.

#define NUM_CAPTN_IDS			   10			// Number of legal attribute identifiers.

//////////////////////////////////////////////////////////////////////////////////////
// Everything below is for internal use only.
#ifdef  SX16_KERNEL

// DMA structures in the core

#define SX_DMA_PAGES    512   // size of buffer
typedef struct
{
    unsigned long  PhysicalAddr;      // physical address of page
    int    Bytes;             // size of page
} SX_DMA_PAGE;

typedef struct
{
    int    hDma;              // handle of dma buffer
    // DA: for changing buffers in program on the fly
    // initial page table is formed by capture_init.
    // When the buffer is activated, THEN the RPS address are changed
    // as(if) required
    int    *rps_base_odd;
    int    *rps_base_even;
    int    *rps_pitch;
    int    *rps_prot_addr;
    int    *rps_ptphys;
    int    *rps_waitcond;
    int    *rps_numlines;

    void  *RpsBufBase;
    void  *RpsAddr;           // RPS buffer
    void  *PtAddr;            // PT buffer
    unsigned long  RpsPhAddr;         // physical address of RPS
    unsigned long  PtPhAddr;          // physical address of PT
    unsigned long  RpsSize; // size of RPS

    void  *UserAddr;          // user's buffer
    int    Bytes;             // size user's of buffer
    int    Pages;             // number of pages user's in buffer
    SX_DMA_PAGE Page[SX_DMA_PAGES];
} SX_DMA;




//////////////////////////////////////////////////////////////////////////////////////

// Queue selection constants.
#define Q_ID_AUD					1			// Audio queue.
#define Q_ID_MUX					2			// Multiplexed queue.
#define Q_ID_VID					3			// Video queue.

// Capture settings: v4l, 616 only.
#define NUM_CAP_INPUTS              2           // Number of carture inputs.
#define NUM_CAP_OUTPUTS             2           // Number of carture outputs.

// Interrupt flags.
#define INTERRUPT_STREAM   0x00000001           // SM interrupt detected.
#define INTERRUPT_VIDEO    0x00000002           // Field interrupt detected.

///////////////////////////////////////////////////////////////////////////////////////////////////




#ifndef WIN32
typedef unsigned long long int __uint64;
#else
typedef unsigned __int64 __uint64;
#endif



typedef struct {
    unsigned short 		SequenceNumber;             // SequenceNumber
    int 				LostPackets;	
    int                 dblBufferCount;
    int                 PacketCount;

    int                 last_error[12];
    long                last_upload_time[2];
    long                last_sync_time[2];
    long                last_packetlost_time[2];
    long                last_dbgtime1[2];
    long                last_dbgtime2[2];
} write_param_t;  // sensoray write params


// OS-independent functions implemented in sx16core.o -----------------------

#ifdef __cplusplus
extern "C"
{
#endif
HBD     	        CORE_board_init 			( unsigned long product_id, unsigned long base_adrs, unsigned long irq, void *dma_adrs_virt, unsigned long dma_adrs_phys );
HBD                 CORE_board_init_debug	( unsigned long product_id, unsigned long base_adrs, unsigned long irq, void *dma_adrs_virt, unsigned long dma_adrs_phys, unsigned long debug_flags );
unsigned long		CORE_attr_get			( HBD board, unsigned long attr_id );
void				CORE_attr_set			( HBD board, unsigned long attr_id, unsigned long value );
unsigned long		CORE_caption_get		( HBD board, unsigned long cap_attr_id );
void				CORE_caption_set		( HBD board, unsigned long cap_attr_id, unsigned long value );

int 				CORE_in_status_get 		( HBD board );

int 				CORE_board_close 		( HBD board );
int 				CORE_board_open 		( HBD board );
void 				CORE_board_reset 		( HBD board );
void 				CORE_board_shutdown		( HBD board );
int					CORE_board_is_516		( unsigned short base_adrs );
unsigned long		CORE_get_stat_KB		( HBD board );
int					CORE_gpio_read			( HBD board );
void 				CORE_gpio_write 		( HBD board, int state );
unsigned short		CORE_microcode_version	( HBD board );
int					CORE_is_dma_set			( HBD board );
unsigned long		CORE_get_dma_remaining	( HBD board );
int					CORE_poll_dma_flag		( HBD board );
int					CORE_reset_interrupt	( HBD board );
unsigned long		CORE_overlay_get_height ( HBD board );
unsigned long		CORE_overlay_get_width	( HBD board );
void				CORE_overlay_set_c1 	( HBD board, unsigned long val );
void				CORE_overlay_set_c2 	( HBD board, unsigned long val );
void				CORE_overlay_set_column ( HBD board, unsigned long val );
void				CORE_overlay_set_data	( HBD board, unsigned long val );
void				CORE_overlay_set_enable	( HBD board, int enable );
void				CORE_overlay_set_row	( HBD board, unsigned long val );
void				CORE_overlay_set_xend	( HBD board, unsigned long val );
void				CORE_overlay_set_xstart ( HBD board, unsigned long val );
void				CORE_overlay_set_yend	( HBD board, unsigned long val );
void				CORE_overlay_set_yfirst ( HBD board, unsigned long val );
void				CORE_overlay_set_ynext	( HBD board, unsigned long val );
void				CORE_overlay_set_ystart ( HBD board, unsigned long val );
unsigned long		CORE_queue_get_any_size	( HBD board, int qid );
void				CORE_start_decode		( HBD board );
void				CORE_start_encode		( HBD board );
void				CORE_stream_bypass		( HBD board, int enable );
int					CORE_stream_is_decoding	( HBD board );
int					CORE_stream_is_encoding	( HBD board );
void				CORE_stream_pause		( HBD board );
int					CORE_stream_read		( HBD board, unsigned char *buffer, unsigned long nbytes );
void				CORE_stream_resume 		( HBD board );
void				CORE_stream_stop		( HBD board );
int					CORE_stream_write		( HBD board, const unsigned char *buffer, unsigned long nbytes );
int 				CORE_get_frame_rate 	( HBD board );
void *				CORE_get_user_data		( HBD board );
__uint64			CORE_get_elapsed_time	( HBD board );
int 				CORE_acquire_mutex		( HBD board );
void 				CORE_release_mutex		( HBD board );
void 				CORE_encoder_out_ctrl	( HBD board, int enable );

#ifndef WIN32
int                 CORE_read_rtp           ( HBD board, unsigned char *buffer );
int                 CORE_rtp_plus           ( HBD board, unsigned char *buffer);
int                 CORE_get_rtp_handle     ( HBD board);
int 				ReadStreamRTPParse		    ( HBD board, int RTP_handle );


// capture functions
int                 CORE_capture_start      ( HBD board, int bMutex );
int                 CORE_capture_stop       ( HBD board); // stops the RPS
int                 CORE_capture_get_status ( HBD board );
int                 CORE_capture_get_SSR     ( HBD board);
int                 CORE_capture_get_PSR     ( HBD board);
int                 CORE_capture_get_MC2     ( HBD board);
int                 CORE_capture_get_MC1     ( HBD board);

int                 CORE_capture_close      ( HBD board );

int                 CORE_capture_init(HBD board, int height, int width, int bpp, int fourcc);
    
int                 CORE_capture_set_fmt    ( HBD board, void *pfmt, int set );
int                 CORE_capture_get_fmt    ( HBD board, void *pfmt );
int                 CORE_capture_set_capbuffer( HBD board, void *val );
int                 CORE_pend_encode        ( HBD board);
int					CORE_get_pend_encode    ( HBD board);

//  sensoray write functions
#ifdef SX16_BUILD_SRAY
unsigned long       SX16CORE_write_sensoray     (HBD board, char *buffer, int skip_data );
unsigned long       SX16CORE_write_sensoray_mux (HBD board, char *buffer, int skip_data );
#endif

int                 CORE_get_write_params   ( HBD board, write_param_t *pWriteParams); 
void                CORE_reset_write_step   ( HBD board );
int                 CORE_get_dma_vformat( HBD board, SX_DMA **ppDma, struct vidcap_pix_format **ppVformat);
#endif

#ifdef __cplusplus
}
#endif
// end functions

///////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////  TYPES  ////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef unsigned char		UCHAR;
typedef unsigned short		USHORT;
typedef unsigned long		ULONG;
typedef long				LONG;
typedef int					BOOL;
typedef unsigned int		UINT;
typedef __uint64			UINT64;
typedef void *				PVOID;


typedef struct {									// CAPTURE BUFFER ---------------
	UCHAR		*capbuf;							//  Pointer to capture (uncompressed) buffer.
	int			capsize;							//  Size of buffer in bytes.
	int			width;								//  Picture size in pixels.
	int			height;
} BUFFER;

typedef struct {									// TEXT OVERLAY POSITION ----------
	int			xstart;
	int			ystart;
	int			xend;
	int			yend;
} OVERLAY_POSITION;

typedef struct {									// TEXT OVERLAY COLORS --------------------
	int			yfirst;								//  Component for first pixel after background
	int			ynext;								//  Component for next pixels after first
	int			c1;									//  Components
	int			c2;
} OVERLAY_COLORS;

typedef struct {									// OVERLAY WRITE PARAMETERS ------------------
	int			row;								//  Text row.
	int			column;								//  Text column.
	int			data;								//  Character data.
} OVERLAY_WRITE;

typedef struct {									// CAPTION PARAMETER BUFFER
	int			window;								//  window #
	USHORT		xstart;								//  window position: X start
	USHORT		xstop;								//  window position: X stop
	USHORT		ystart;								//  window position: Y start
	USHORT		ystop;								//  window position: Y stop
	USHORT		scale;								//  font scale
	int			color;								//  caption color
	int			transp;								//  transparency attribute
	char		text[80];							//  caption text
	BOOL		on; 								//  window on/off
} BUF_CAPTN_PARAM;

typedef struct {									// CONFIGURATION REGISTER IMAGE --------------
	UCHAR		address;							//  Register address.
	UCHAR		data;								//  Register's data image.
} REGSET;

struct VPAR;	// forward declaration

typedef struct {									// PARAMETER ATTRIBUTES ----------
	void	( *change_anytime )( struct VPAR * );	//   Pointer to function that changes parameter while streaming (or 0 if changes not allowed during streaming).
	ULONG	min_legal_value;						//   Minimum legal value.
	ULONG	max_legal_value;						//   Maximum legal value.
	ULONG	default_value;							//   Default value.
} ATTR_ITEM;

typedef struct {									// CAPTION PARAMETERS  ----------
	ULONG	min_legal_value;						//   Minimum legal value.
	ULONG	max_legal_value;						//   Maximum legal value.
	ULONG	default_value;							//   Default value.
} CAPTN_PARAM;

enum PARM_ACTION {									// TYPE CLASSIFICATION FOR SM/ENCODER/DECODER/ACODEC PARAMETER (DON'T CHANGE THESE !!!) -------------
	VAL_DEFAULT = 0,								//   Set value of module-level default image.
	VAL_RUNTIME = 1									//   Set value of runtime image.
};

enum STREAM_STATE {									// STATE OF MPEG STREAM ----------------------
	STREAM_HALTED	= 0,							//   Shut down.
	STREAM_RUNNING,									//   Running.
	STREAM_PAUSED,									//   Paused.
	NUM_STREAM_STATES								// Number of STREAM_STATEs.
};



// temp buf size for sensoray writes.

#define TEMP_BUF_SIZE		    (40 * 1024)
#define TEMP_BUF_SIZE_LARGE	    (500 * 1024)
#define TEMP_BUF_SIZE_LARGER	    (900 * 1024)
#define MIN_WRITE_SIZE_MUX         1024

#define PACHEADER_SIZE      (3*1024)
#define FIFO_SIZE           (3*1024)

// constants
#define MAX_SMRSPSIZ		17					// Maximum size of SM status packet in words.
#define	MPEG_NONE			0xFF				// When active_microcode is set to this, indicates SM has no uploaded microcode.


typedef struct
{
    char *capbuf;       // DMA capture buffer (currently unused.  UserAddr is used for DMA)
    SX_DMA capdma;      // DMA structure for capture
    int caprps;         // DMA RPS address for capture
} cap_dma_par_t;


typedef struct VPAR {								// BOARD OBJECT ------------------
													//  Constant attributes:
	int					board_num;					//	  Board number.
	ULONG					base_adrs;					//    Logical (and physical, in case of 516) base address of primary application register block.
	int					irq;						//    Boards interrupt number.
	int					product_id;					//    Product id, i.e. 516 or 616.
													//  Model-dependent constants:
	int					i2caddr_dec;				//    SAA7113 decoder's I2C device sub-address.
	int					i2caddr_enc;				//    SAA7122 encoder's I2C device sub-address.
	int					L3mask_data;				//    L3 data bit mask.
	int					L3mask_mode;				//    L3 mode bit mask.
	int					L3mask_clk;					//    L3 clock bit mask.
	int					L3bit_data;					//    L3 data bit number.
	ATTR_ITEM			*attr;						//    Pointer to constant parameter attributes for this model.
	CAPTN_PARAM			*captn_prm;					//    Pointer to constant caption parameter for this model.
													//  Board state:
	BOOL				is_open;					//    Board is open.
													//  Error handling:
	int					errflags;					//    Error logging flags.
	ULONG				debug_flags;				//    Debug message control flags.
													//  Stream Machine state:
	int					active_microcode;			//    Currently loaded microcode: MPEG_RECORD, MPEG_PLAY or MPEG_NONE.
	USHORT				microcode_version;			//    Version number of the currently loaded microcode.
	USHORT				sm_responses[MAX_SMRSPSIZ];	//    Buffer to hold largest possible SM status packet.
	ULONG				sm_response_expected_size;	//    Anticipated word count to follow the first word of SM status response packet.
	ULONG				sm_response_length;			//    Length of SM status response packet in words.
	enum STREAM_STATE	stream_state;				//	  SM's mpeg stream state: HALTED, RUNNING, or PAUSED.
//	BOOL				poll_enabled;				//    MPEG data may be transferred to/from the sm. 
	UINT				num_dma_bytes;				//    Number of mpeg bytes transferred since current sm dma transfer began.
													//  Text overlay control:
	OVERLAY_POSITION	overlay_position;			//    Position.
	OVERLAY_COLORS		overlay_colors;				//    Color.
	OVERLAY_WRITE		overlay_write;				//    Write parameters.
													//  Caption parameters as viewed by module and application layers:
	ULONG				captn_default[NUM_CAPTN_IDS];	// Default parameters, set by module init.  These are treated as constants by the core.
	ULONG				captn_current[NUM_CAPTN_IDS];	// Current parameters, loaded from defaults at Open, used/modified until closed.
	char				captn_txt_current[80];			// Current parameters: caption text to update.
	BUF_CAPTN_PARAM		captn_parameters[2];			// 0 - for window #1;	1 - for window #2.
													//  Generic operating parameters as viewed by module and application layers:
	ULONG				attr_default[NUM_ATTR_IDS];	//    Default parameters, set by module init.  These are treated as constants by the core.
	ULONG				attr_current[NUM_ATTR_IDS];	//    Current parameters, loaded from defaults at Open, used/modified until closed.
													//  Statistics:
	ULONG				KB_transferred;				//    Number of mpeg KB transferred since mpeg encoder/decoder started.
													//  516-specific values:
	int					lastaddr;					//    Last address programmed into GA's indirect register adrs reg.
	int					saved_lastaddr;				//    Saved lastaddr for foreground/background context switches.
													//  DMA buffer:
	UCHAR				*dmabuf_virt;				//    Virtual address.
	ULONG				dmabuf_phys;				//	  Physical (DMA) address.
													//  Encoder decoder start time.
	UINT64				stream_start_time;			//    Time in milliseconds.
													//  Mutex object
	PVOID				mutex_object;				//    Harware mutex.
    PVOID               i2c_lock;                   // i2c mutex
	int                 b_upload;                   // uploading firmware flag
    int                 cap_frozen;                // capture frozen

#ifndef WIN32
#if defined (SX16_BUILD_SRAY)

    struct RTP_INFO      rtp;                        // RTP information see rtp.h
#endif
#endif

    // capture dma parameters
    cap_dma_par_t      vpar_dma_cap;
    // clipping dma parameters
    cap_dma_par_t      clip_dma_cap;
    // capture parameters/format
    struct vidcap_pix_format  cap_pixfmt;
    // video memory overlay parameters/format
    struct vidcap_pix_format  ovr_pixfmt;

    int                 overlay_started;            //video overlay start/stop status
    int                 capture_started;  
    
    int                 clip_started;               //video clipping start/stop status
#ifndef WIN32
    // sensoray write, sensoray write mux buffers
    unsigned char       *TempBuf;                   //temp buffer for mux sream parsing during playback
    unsigned char       *TempBufEnd;	            //end of temp buffer
    int                 TempBufSize;
    int                 TempBufHW; // high water to flush(ideally never have to do this, but necessary on 301)
    unsigned char       *fifo;
    int                 fifo_len;

    unsigned char       *PacHdr;
    int                 pachdr_len;

    int                 bFlushed; // if buffer was flush before sync
    unsigned short 		SequenceNumber;             // SequenceNumber
    int 				LostPackets;
    unsigned long 		PacketCount;	
    unsigned long       dblBufferCount;
    long                last_sync_time[2];
    long                last_upload_time[2];
    long                last_lostpacket_time[2];
    long                last_dbgtime1[2];
    long                last_dbgtime2[2];
    int                 last_err[12]; 
    unsigned int        write_step;                // sensoray write step
    unsigned int        write_step_mux;            // sensoray write state mux
#endif

    int bOvlInit;
    int bCapInit;
	int pend_encode;
    //  Virtual function pointers for model-dependent functions:
	void	( *context_save )		( struct VPAR * );
	void	( *context_restore )	( struct VPAR * );
	void	( *dec_config )			( struct VPAR * );
	void	( *enc_config )			( struct VPAR * );
	void	( *destructor )			( struct VPAR * );
	int		( *gpio_read )			( struct VPAR * );
	void	( *gpio_write )			( struct VPAR * , int );
	BOOL	( *i2c_is_busy )		( struct VPAR * );
	int		( *i2c_read )			( struct VPAR * , int, int );
	void	( *i2c_write )			( struct VPAR * , int, int, int );
	int		( *L3_read )			( struct VPAR * );
	void	( *L3_write )			( struct VPAR * , int );
	void	( *set_stream_bypass )	( struct VPAR * , int );
	void	( *set_video_in )		( struct VPAR * );
	void	( *sm_mpeg_read )		( struct VPAR * , USHORT *, UINT );
	void	( *sm_mpeg_write )		( struct VPAR * , USHORT *, UINT );
	void	( *sm_reg_block_write )	( struct VPAR * , int, USHORT *, int );
	USHORT	( *sm_reg_read )		( struct VPAR * , int );
	void	( *sm_reg_write )		( struct VPAR * , int, USHORT );
	void	( *sm_reset )			( struct VPAR * );
	void	( *captn_pos_calc )		( struct VPAR * , void * );
	void	( *captn_param_load )	( struct VPAR * , void *, ULONG action_id );
	void	( *enable_irq )			( struct VPAR * );
	void	( *disable_irq )		( struct VPAR * );
	void	( *shutdown_irq )		( struct VPAR * );
	int dma;
} VPAR, *PVPAR;

// debug and error info
#define IDX_INFO_FW     0
#define IDX_ERR_PKTHDR  1  // if PES length field(extra bytes) not zero
#define IDX_ERR_COREWR  2
#define IDX_ERR_TEMPOVL 3
#define IDX_ERR_SEARCH1 4
#define IDX_ERR_SEARCH2 5
#define IDX_ERR_UNUSED0 6 
#define IDX_ERR_UNUSED1 7
#define IDX_INFO_PREFIX 8
#define IDX_ERR_TEMPREF 9
#define IDX_ERR_PICTYPE 10

///////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////  MANIFEST CONSTANTS  ////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

// Generic parameter min/max limits -------------------------------------------------

// Video bit rate limits.
// minimum bit rate is 1 Mbps, but allow for lower setting
// Max rate 15 Mbps
#define BITRATE_MIN			50000				// Minimum legal bit rate in bps.
#define BITRATE_MAX			15000000			// Maximum legal bit rate in bps.


// Command opcodes.
#define SM_STOP_CMD         0x0000				// Stop stream.
#define SM_START_CMD        0x0001				// Start stream.
#define SM_PAUSE_CMD        0x0002				// Pause stream.
#define SM_RESUME_CMD		0x0003				// Resume stream.
#define SM_NOP				0xFFFF				// Indicates that no SM command is to be invoked (not an actual SM opcode).

// Timeouts, in milliseconds, for SM ack response.
#define	SM_NEXT_WORD_TOUT	20					// Max time (for each word) to receive SM status response words 2 through N.
#define	SM_PARAM_ACK_TOUT	500					// Parameters change.
#define	SM_START_TOUT		500					// Start Stream command ack timeout.
#define	SM_STOP_TOUT		500					// Stop Stream command ack timeout.
#define	SM_PAUSE_TOUT		500					// Pause Stream command ack timeout.
#define	SM_RESUME_TOUT		500					// Resume Stream command ack timeout.
#define	SM_UCODE_ACK_TOUT	500					// Microcode upload.

// Bit masks for SM_INTCTRL_REG.
#define SMICTL_DMA_ENABLE	0x1000				// DMA request.
#define SMICTL_DFIFO_FULL	0x0080				// DRAM FIFO full.
#define SMICTL_DFIFO_EMPTY	0x0040				// DRAM FIFO empty.
#define SMICTL_RCFIFO_EMPTY	0x0008				// RC FIFO (command invocation fifo) is empty.
#define SMICTL_TXFIFO_FULL	0x0004				// TX FIFO (response/status fifo) is full.
#define SMICTL_TXFIFO_NOTMT	0x0002				// TX FIFO (response/status fifo) is not empty.
#define SMICTL_DMA_DONE		0x0001				// DMA done.

// Bit masks for SM_INTINFO_REG.
#define SMSTAT_DMA_REQUEST	0x1000				// DMA request is asserted.
#define SMSTAT_DFIFO_FULL	0x0080				// DRAM FIFO full.
#define SMSTAT_DFIFO_EMPTY	0x0040				// DRAM FIFO empty.
#define SMSTAT_RCFIFO_EMPTY	0x0008				// RC FIFO (command invocation fifo) is empty.
#define SMSTAT_TXFIFO_FULL	0x0004				// TX FIFO (response/status fifo) is full.
#define SMSTAT_TXFIFO_NOTMT	0x0002				// TX FIFO (response/status fifo) is not empty.
#define SMSTAT_DMA_DONE		0x0001				// DMA done.

// Parameter registers, indirectly written to SM through portal register SM_PARAM_REG.
#define SMP_AUDIO_BITRATE	0x00				// Audio bit rate.  See SM docs for details.
#define SMP_AUDIO_FORMAT	0x01				// Audio encoding format: 0=MPEG1 layer 1, 1=MPEG1 layer2, 2=MPEG1 layer ?, 3=Dolby digital (AC3) 2-channel.
#define SMP_STEREO_MODE		0x02
#define SMP_AUDIO_SMPRATE	0x03				// Audio ( SampRate<3:0>, PcmRes<3:0> ). SmpRate: 0=44.1 KHz, 1=48, 2=32. PcmRes: 0=16-bit, 1=18, 2=20, 3=24.
#define SMP_RESERVED_04		0x04
#define SMP_AUDIO_ADVANCED	0x05
#define SMP_RESERVED_06		0x06
#define SMP_RESERVED_07		0x07
#define	SMP_FILTER			0x08
#define SMP_VIDEOCTRL		0x09				// Video control: ( 0, NTSC, PAL, TopFldFirst, 601/656, 0, Loopback, 0 ).
#define SMP_EVEN_OFFSET		0x0A
#define SMP_ODD_OFFSET		0x0B
#define	SMP_RESOLUTION		0x0C				// Resolution/scaling control.
#define SMP_H_YC_OFFSET		0x0D
#define SMP_RESERVED_0E		0x0E
#define SMP_GOPLEN_IPDIST	0x0F

#define	SMP_VIDEORATELSB	0x10
#define	SMP_VIDEORATEMSB	0x11
#define SMP_MOTION_SEARCH	0x12
#define	SMP_VBRMODE			0x13				// Variable bit rate control: ( 0000000, VBR_enable ).
#define SMP_RESERVED_14		0x14
#define SMP_VIDEO_MISC		0x15
#define SMP_RESERVED_16		0x16
#define SMP_RESERVED_17		0x17
#define SMP_TEMPORAL_SEL	0x18
#define SMP_TEMPORAL_THRESH	0x19
#define SMP_RESERVED_1A		0x1A
#define SMP_RESERVED_1B		0x1B
#define SMP_AV_SELECT		0x1C				// ( SectorHeaderEnable, AVselect<1:0>, NavMsgEnable, reserved<3:0> ).
#define SMP_MUX_FORMAT		0x1D				// Multiplex format: SVCD, VCD, DVD, etc.
#define SMP_RESERVED_1E		0x1E
#define SMP_PROGRAM_NUM		0x1F

#define SMP_CUSTOM_MUX_20	0x20
#define SMP_CUSTOM_MUX_21	0x21
#define SMP_CUSTOM_MUX_22	0x22
#define SMP_CUSTOM_MUX_23	0x23
#define SMP_INITIAL_SCR_24	0x24
#define SMP_INITIAL_SCR_25	0x25
#define SMP_INITIAL_SCR_26	0x26
#define SMP_INITIAL_SCR_27	0x27
#define SMP_INITIAL_PTS_28	0x28
#define SMP_INITIAL_PTS_29	0x29
#define SMP_INITIAL_PTS_2A	0x2A
#define SMP_INITIAL_PTS_2B	0x2B
#define SMP_SECTOR_CTRL		0x2C
#define SMP_SYSTEM_FORMAT	0x2D
#define SMP_DMA_SIZE_LSB	0x2E
#define SMP_DMA_SIZE_MSB	0x2F

#define SMP_RESERVED_30		0x30
#define SMP_RESERVED_31		0x31
#define SMP_RESERVED_32		0x32
#define SMP_RESERVED_33		0x33
#define SMP_RESERVED_34		0x34
#define SMP_MENU_CTRL		0x35
#define SMP_RESERVED_36		0x36
#define SMP_RESERVED_37		0x37

#define SMP_PARAM_CTRL		0xB8				// Parameter control register: MUST write "1" to this after uploading new params.

// SAA7146 constants (model 616 only) ------------------------------------------------------------------

// Register addresses, accessed in PCI region 0.

/* start 616 capture specific functions */
#define P_BASEODD1          0x000
#define P_BASEEVEN1         0x004
#define P_PROTADDR1         0x008
#define P_PITCH1            0x00c
#define P_PAGEMAP1          0x010
#define P_FORMAT1           0x014
#define P_BASEODD2          0x018
#define P_BASEEVEN2         0x01c
#define P_PROTADDR2         0x020
#define P_PITCH2            0x024
#define P_PAGEMAP2          0x028
#define P_FORMAT2           0x02c
#define P_BASEODD3          0x030
#define P_BASEEVEN3         0x034
#define P_PROTADDR3         0x038
#define P_PITCH3            0x03c
#define P_PAGEMAP3          0x040
#define P_FORMAT3           0x044
#define P_PCIBTV            0x048
#define P_D1SYNC            0x050
#define P_D1VIDEO           0x054
#define P_BRSCTL            0x058
#define P_HPSCTL            0x05c
#define P_HPSVS             0x060
#define P_HPSVSG            0x064
#define P_HPSHPS            0x068
#define P_HPSHFS            0x06c
#define P_BCSCTL            0x070
#define P_CKEY              0x074
#define P_HPSFORM           0x078
/* end 616 capture specific functions           */
#define P_DEBICFG			0x007C				// DEBI configuration.
#define P_DEBICMD			0x0080				// DEBI command.
#define P_DEBIPAGE			0x0084				// DEBI page.
#define P_DEBIAD			0x0088				// DEBI address.
#define P_I2CCTRL			0x008C				// I2C control.
#define P_I2CSTAT			0x0090				// I2C status.
#define P_LCTS0             0x00cc
#define P_LCTS1             0x00d0
#define P_RPSTO0            0x00d4
#define P_RPSTO1            0x00d8
#define P_INTEN				0x00DC				// Interrupt enable.
#define P_GPIO				0x00E0				// General-purpose I/O.
#define P_MC1				0x00FC				// Master control 1.
#define P_MC2				0x0100				// Master control 2.
#define P_RPSADDR0          0x0104
#define P_RPSADDR1          0x0108
#define P_ISR				0x010C				// Interrupt status.
#define P_PSR				0x0110				// Primary status.
#define P_SSR				0x0114				// Secondary status.




// Interrupt masks.
#define INT_OFF				0x00000000			// INTEN mask: disable all interrupts.

// MC1 masks.
#define MC1_SOFT_RESET		0x80000000			// SAA7146 software reset.  active low
#define MC1_DEBI_ENABLE		0x08000800			// Enable DEBI pins.
#define MC1_I2C_ENABLE		0x01000100			// Enable I2C pins.
#define MC1_TR_E_DEBI		0x00800080			// DEBI transfer enable.

// I2C constants.
#define I2C_ATTRNOP			0					// NOP attribute.
#define I2C_ATTRSTOP		1					// Stop attribute.
#define I2C_ATTRCONT		2					// Continue attribute.
#define I2C_ATTRSTART		3					// Start attribute.

#define I2C_BUSY_616		0x00000001			// I2CCTRL mask: I2C busy.
#define I2C_ABORT			0x00000080			// I2CSTAT mask: abort transaction.
#define I2C_CLKSEL			0x00000400			// I2C bit rate = PCIclk/480 = 68.75 KHz.

												// Model 616 rd/wr command mask for I2C control register:
#define I2C_WR_616			0x00000000			//   Write command.
#define I2C_RD_616			0x01000000			//   Read command.

// MC2 upload masks.
#define MC2_DEBI_UPLD_CMD	0x00020002			// DEBI upload command.
#define MC2_I2C_UPLD_CMD	0x00010001			// I2C upload command.

#define MC2_DEBI_UPLD_DONE	0x00000002			// DEBI upload done.
#define MC2_I2C_UPLD_DONE	0x00000001			// I2C upload done.

// PSR masks.
#define PSR_DEBI_BUSY		0x00080000			// DEBI is busy or halted by an error.

// Interrupt bits.
#define ISR_GPIO0			(1<<3)			    // SAA7146 GPIO0 interrupt bit.
#define ISR_GPIO1			(1<<4)			    // SAA7146 GPIO1 interrupt bit.
#define ISR_GPIO2			(1<<5)			    // SAA7146 GPIO2 interrupt bit.
#define ISR_GPIO3			(1<<6)			    // SAA7146 GPIO3 interrupt bit.
#define ISR_FIDB			(1<<7)			    // SAA7146 Field ID port B interrupt bit.
#define ISR_FIDA			(1<<8)			    // SAA7146 Field ID port A interrupt bit.

#define ISR_RPS_I0			(1<<27)			    // SAA7146 RPS0 interrupt bit.
#define ISR_RPS_I1			(1<<28)			    // SAA7146 RPS1 interrupt bit.

// DEBI page table constants.
#define DEBIPAGE_ENABLE		0x00000800			// DEBIPAGE mask: enable DEBI paging.
#define DEBIPAGE_DISABLE	0x00000000			// DEBIPAGE mask: disable DEBI paging.

// Composite image for DEBI paging.
#define DEBIPAGE_TABLE(ENABLE,TBLADRS)		( (ENABLE) | ( (TBLADRS) << 12 ) )

// DEBI command opcodes.
#define DEBICMD_OPCODE_RD 	0x00010000			// Read.
#define DEBICMD_OPCODE_WR 	0x00000000			// Write.

// Composite DEBI constants.
#define DEBICMD_NBYTES(NBYTES)		( (NBYTES) << 17 )
#define DEBICMD_READ(ADRS,NBYTES)	( DEBICMD_OPCODE_RD | DEBICMD_NBYTES(NBYTES) | ( (ADRS) & 0xFFFF ) )
#define DEBICMD_WRITE(ADRS,NBYTES)	( DEBICMD_OPCODE_WR | DEBICMD_NBYTES(NBYTES) | ( (ADRS) & 0xFFFF ) )

// DEBI configuration register masks, used to communicate with the SM.
#define DEBICFG_FAST		0x10000000			// Enable fast mode.
#define DEBICFG_SLAVE16		0x00080000			// Slave supports 16-bit transfers.
#define DEBICFG_INCREMENT	0x00040000			// Auto address increment during block transfers.
#define DEBICFG_INTEL		0x00020000			// Intel-mode operation.
#define DEBICFG_TMR_DISABLE	0x00010000			// Disable timer.
#define DEBICFG_TIMEOUT(N)	( (N) << 22 )		// Timeout in PCI clocks.
#define DEBICFG_SWAP(N)		( (N) << 20 )		// Byte lane steering.

// Composite images for DEBI configuration register.
#define DEBICFG_BASIC		( DEBICFG_SLAVE16 | DEBICFG_INTEL | DEBICFG_TIMEOUT(1)/* | DEBICFG_FAST*/ )	// 2 PCI clocks allowed for SM to assert RDY at start of access.
#define DEBICFG_REG_ACCESS	( DEBICFG_BASIC | DEBICFG_SWAP(0) )		// Standard byte lane steering for accessing the SM's control registers.
#define DEBICFG_MPEG_ACCESS	( DEBICFG_BASIC | DEBICFG_SWAP(1) )		// Modified byte lane steering for exchanging mpeg data with the SM.

// Register addresses accessed via gate array (model 516 only) ---------------------------------------------------------------------

												// Directly accessed internal gate array registers:
#define G_DATAIO			0x0000				//   data at location selected by G_INDEX.
#define G_INDEX				0x0002				//   Index register used for indirectly accessing all registers above 0x0006.
#define G_MEMORYADDRESS		0x0004				//   Memory mapped registers base address.
#define G_SIGNATURE			0x0006				//   Board signature.

												// Indirectly accessed internal gate array registers:
#define G_STATUS			0x0000				//   Status:            ( 0, 0, 0, GPIO5, GPIO3, GPIO2, GPIO1, O_FID, I_FID, RCV2, RCV1, RTS1, RTS0, ENDEC, DREQ, SMINT ).
#define G_EVENTFLAGS		0x0002				//   Event flags:       ( 0, 0, 0,     0,     0,     0,     0, O_FID, I_FID, RCV2, RCV1, RTS1, RTS0, ENDEC, DREQ, SMINT ).
#define G_IRQENABLE			0x0004				//   Interrupt enables: ( 0, 0, 0,     0,     0,     0,     0, O_FID, I_FID, RCV2, RCV1, RTS1, RTS0, ENDEC, DREQ, SMINT ).
#define G_IRQSELECT			0x0006				//   IRQ select: 3, 5, 7, 10, 11 or 15.
#define G_AUDIOCONTROL		0x0008				//   Audio control.
#define G_L3CONTROL			0x000A				//   L3 control.
#define G_GPIO				0x000C				//   General-purpose digital I/O.
#define G_MISC				0x000E				//   Misc. control: ( 0x000, BSWAP, OPTION2, OPTION1, LED ).
#define G_CAPRAMDATA		0x0010				//   Caption RAM data.
#define G_CAPRAMINDEX		0x0012				//   Caption RAM index.
#define G_CAPWINPOSN_1X		0x0014				//   Caption window #1 X position.
#define G_CAPWINPOSN_1Y		0x0016				//   Caption window #1 Y position.
#define G_CAPWINPOSN_2X		0x0018				//   Caption window #2 X position.
#define G_CAPWINPOSN_2Y		0x001A				//   Caption window #2 Y position.
#define G_CAPWINSCALE_1		0x001C				//   Caption window 1 scale.
#define G_CAPWINSCALE_2		0x001E				//   Caption window 2 scale.
#define G_CAPRAMCHAR_1		0x0000				//   Caption character RAM base for char  0 - 31.
#define G_CAPRAMCHAR_2		0x0100				//   Caption character RAM base for char 32 - 63.
#define G_CAPRAMATTR_1		0x0200				//   Caption window 1 attr RAM base.
#define G_CAPRAMATTR_2		0x0240				//   Caption window 2 attr RAM base.
#define G_I2CDEVICEADDR		0x0020				//   I2C device address.
#define G_I2CSUBADDR		0x0022				//   Sub-address within I2C device.
#define G_I2CDATA			0x0024				//   I2C data.
#define G_I2CCONTROL		0x0026				//   I2C control/status.
#define G_RESETCONTROL		0x0028				//   Reset/loopback control: ( Loopback, 00000000000, PLLRST, HARDRST, APLLRST, VIDRST ).

// Gate array (model 516 only) interrupt masks
#define GI_SMINT			0x0001				// Interrupt mask bit for SM interrupt.

// SM control register addresses on local SM access bus -------------------------------------
												// Base address of SM control registers on SM local bus interface.
#define SMBASE_516			0x0080				//   516: accessed through GA indirect mechanism.
#define SMBASE_616			0x0000				//   616: accessed via DEBI.

												// Address offsets of SM control registers, with respect to associated SMBASE.
#define SM_FIFOTX_REG		0x0006				//   Stream TX FIFO (16x16), read only.
#define SM_FIFORX_REG		0x0008				//   Stream RC FIFO (16x16), write only.
#define SM_INTCTRL_REG		0x000A				//   Interrupt enable and control.
#define SM_INTINFO_REG		0x000C				//   Interrupt info.
#define SM_DMAINFO1_REG		0x000E				//   DMA info-1.
#define SM_DMAINFO2_REG		0x0010				//   DMA info-2.
#define SM_TRANSAC_REG		0x0016				//   BS (byte stream) transaction register, counts number of 8-byte transactions.
#define SM_MICROCODE_REG	0x001C				//   Microcode download portal.
#define SM_VQSIZE_LSW_REG	0x0020				//   VideoQueueSize[15:0]:  data size in video queue.
#define SM_VQSIZE_MSW_REG	0x0022				//   VideoQueueSize[19:16]: data size in video queue.
#define SM_MQSIZE_LSW_REG	0x0024				//   MuxQueueSize[15:0]:  data size in multiplexed (video + audio) queue.
#define SM_MQSIZE_MSW_REG	0x0026				//   MuxQueueSize[19:16]: data size in multiplexed (video + audio) queue.
#define SM_AQSIZE_REG       0x0028				//   AudioQueueSize: data size in audio queue.
#define SM_DMAMASTER_REG	0x002A				//   DMA Master control.
#define SM_ENDIAN_REG		0x0038				//   Endian configuration control.
#define SM_PARAM_REG        0x0040				//   Configuration Parameters download portal.

#define SM_DRAM_REG_516		0x0800				//   0x0800 to 0x0FFF : DRAM data window (516).
#define SM_DRAM_REG_616		0x8000				//   0x8000 to 0x8FFF : DRAM data window (616).

// I2C sub-addresses for devices that are accessed via I2C interface -------------------

												// Devices on the 516:
#define I2CADDR_DEC_516		0x4A				//   SAA7113 decoder.
#define I2CADDR_ENC_516		0x88				//   SAA7122 encoder.

												// Devices on the 616:
#define I2CADDR_DEC_616		0x42				//   SAA7114 decoder.
#define I2CADDR_ENC_616		0x88				//   SAA7121 encoder.
#define I2CADDR_IO_616		0x40 				//   PCA9554 I2C I/O port.
#define I2CADDR_OVR_616		0x70 				//   GA overlay I2C I/O port.

// Register addresses for PCA9554 I2C 8-bit I/O port, which is present only on the 616.
// The audio control bus bit-bang register is indirectly accessed over the I2C bus via the
// PCA9554 device, which in turn provides access to an 8-bit general-purpose port via I2C interface.
// The addresses are left-shifted by 1 bit to align with address field in first byte sent to device.

#define PCA_PORT_IN			0					// Read port.
#define PCA_PORT_OUT		1					// Write port.
#define PCA_PORT_CONFIG		3					// Data direction control.

// PCA bit positions and masks, except for the audio L3 bits.

#define PCA_BIT_DOUT		2					// Bit position of general-purpose digital outputs in PCA byte.
#define PCA_BIT_DIN			0					// Bit position of general-purpose digital inputs in PCA byte.

#define PCA_MASK_DOUT		( 3 << PCA_BIT_DOUT )
#define PCA_MASK_DIN		( 3 << PCA_BIT_DIN )

// Decoder register I2C subaddresses.

#define D_CHIP_VERSION		0x00
#define D_INCREMENT_DELAY	0x01
#define D_AIN_CTRL_1		0x02
#define D_AIN_CTRL_2		0x03
#define D_AIN_CTRL_3		0x04
#define D_AIN_CTRL_4		0x05
#define D_HSYNC_START		0x06
#define D_HSYNC_STOP		0x07
#define D_SYNC_CONTROL		0x08
#define D_LUM_CTRL			0x09
#define D_LUM_BRIGHTNESS	0x0A
#define D_LUM_CONTRAST		0x0B
#define D_CHROMA_SAT		0x0C
#define D_CHROMA_HUE		0x0D
#define D_CHROMA_CTRL		0x0E
#define D_CHROMA_GAIN		0x0F
#define D_CHROMA_GAIN2      0x10
#define D_FORMAT_DELAY_7114	0x11
#define D_OUT_CTRL_1_7114	0x12
#define D_OUT_CTRL_2_7114	0x13

#define D_FORMAT_DELAY_7113	0x10
#define D_OUT_CTRL_1_7113   0x11
#define D_OUT_CTRL_2_7113	0x12
#define D_OUT_CTRL_3_7113	0x13

//#define D_OUT_CTRL_3		0x13
#define D_RESERVED_14		0x14
#define D_V_GATE1_START		0x15
#define D_V_GATE1_STOP		0x16
#define D_V_GATE1_MSB		0x17

#define D_STATUS			0x1F

// Encoder register I2C subaddresses.

#define E_WIDE_SCREEN_LSB	0x26
#define E_WIDE_SCREEN_MSB	0x27
#define E_RTC_BURSTSTART	0x28
#define E_BURST_END			0x29
#define E_COPYGUARD_ODD_0	0x2A
#define E_COPYGUARD_ODD_1	0x2B
#define E_COPYGUARD_EVEN_0	0x2C
#define E_COPYGUARD_EVEN_1	0x2D
#define E_COPYGUARD_ENABLE	0x2E

#define E_INPUTCTRL			0x3A

#define E_CHROMA_PHASE		0x5A
#define E_GAIN_U_LSB		0x5B
#define E_GAIN_V_LSB		0x5C
#define E_GAIN_U_MSB		0x5D
#define E_GAIN_V_MSB		0x5E
#define E_CCR_BLANKING_LVL	0x5F

#define E_NULL_60			0x60
#define E_STANDARD_CTRL		0x61
#define E_RTCEN_BURSTAMPL	0x62
#define E_SUBCARRIER_0		0x63
#define E_SUBCARRIER_1		0x64
#define E_SUBCARRIER_2		0x65
#define E_SUBCARRIER_3		0x66
#define E_LINE21_ODD_0		0x67
#define E_LINE21_ODD_1		0x68
#define E_LINE21_EVEN_0		0x69
#define E_LINE21_EVEN_1		0x6A
#define E_RCV_PORT_CTRL		0x6B
#define E_TRIG_CTRL_LSB		0x6C
#define E_TRIG_CTRL_MSB		0x6D
#define E_MULTI_CTRL		0x6E
#define E_CC_ENABLE			0x6F

#define E_RCV2_OUT_START	0x70
#define E_RCV2_OUT_END_LSB	0x71
#define E_RCV2_OUT_END_MSB	0x72
#define E_TTX_REQ_H_START	0x73
#define E_TTX_REQ_H_DELAY	0x74
#define E_VSYNC_SHIFT		0x75
#define E_TTX_ODD_REQ_V_S	0x76
#define E_TTX_ODD_REQ_V_E	0x77
#define E_TTX_EVEN_REQ_V_S	0x78
#define E_TTX_EVEN_REQ_V_E	0x79
#define E_FIRST_ACTIVE_LINE	0x7A
#define E_LAST_ACTIVE_LINE	0x7B
#define E_MSB_VERTICAL		0x7C
#define E_NULL_7D			0x7D					// This location MUST be programmed on the SAA7121, but apparently not req'd on SAA7120.
#define E_DISAB_TTXLINE_LSB	0x7E
#define E_DISAB_TTXLINE_MSB	0x7F

// Audio codec interface ------------------------------------

// Register sub-addresses.
#define A_VOLUME			0x00					// Volume level.
#define A_BASS_TREBLE		0x01					// Bass and audio_treble levels.
#define A_MUTE_ENABLE		0x02					// Mute enable.

// Target registers within the audio codec.
#define L3REG_DATA0			0x00
#define L3REG_DATA1			0x01
#define L3REG_STATUS		0x02

// L3 Control Register bit numbers.
													// Model 516:
#define L3BIT_DATA_516		2						//   Data stream.
#define L3BIT_MODE_516		1						//   Mode control.
#define L3BIT_CLK_516		0						//   Transfer clock.
													// Model 616:
#define L3BIT_DATA_616		4						//   Data stream.
#define L3BIT_MODE_616		5						//   Mode control.
#define L3BIT_CLK_616		6						//   Transfer clock.

#define L3MASK_DATA_616		( 1 << L3BIT_DATA_616 )
#define L3MASK_MODE_616		( 1 << L3BIT_MODE_616 )
#define L3MASK_CLK_616		( 1 << L3BIT_CLK_616 )

#define L3MASK_616			( L3MASK_DATA_616 | L3MASK_MODE_616 | L3MASK_CLK_616 )

// Audio codec's fixed address on the 516's bit-banging bus.

#define ADDR_AUDIOCODEC		5

// Register definitions for the audio codec.
#define SEL_STATUS_0		0x00					// Status/control register 0.
#define SEL_STATUS_1		0x80					// Status/control register 1.

#define SEL_DATA0_VOLUME	0x00					// Volume level control.
#define SEL_DATA0_BASSTREB	0x40					// Bass/audio_treble level controls.
#define SEL_DATA0_MISC		0x80					// Mute, other misc. stuff.
#define SEL_DATA0_EXTADDR	0xC0					// Select extended register.
#define SEL_DATA0_EXTDATA	0xE0					// Window to selected extended register.

#define DATA0_EXT_MA		0x00					// Mixer gain channel 1.
#define DATA0_EXT_MB		0x01					// Mixer gain channel 2.
#define DATA0_EXT_MS_MM		0x02					// MIC sensitivity<2:0>, mixer mode<1:0>.
#define DATA0_EXT_AG_IGLO	0x04					// AGC control, input amplifier gain<1:0>.
#define DATA0_EXT_IGHI		0x05					// Input amplifier gain<6:2>.
#define DATA0_EXT_AT_AL		0x06					// AGC time constant<2:0>, AGC output level<1:0>.

// Status/control register 0 bit masks.
#define STAT0_CONFIG		0x20					// Fixed configuration: SysClk = 256f, DataFormat = I2C, DCFilter = off.
#define STAT0_RESET_ON		0x40					// Assert device reset.
#define STAT0_RESET_OFF		0x00					// Negate device reset.

// Gate array register masks ----------------------------

// I2C control register bit masks.
#define I2C_START			0x0001					// Start transaction.
#define I2C_BUSY_516		0x0002					// Transaction in progress.
#define I2C_ACK_516			0x0004

// Reset Control register bits.
#define RST_LOOPBACK		0x8000					// Video loopback enable.
#define RST_VPLL			0x0008					// SM video PLL reset.
#define RST_HARD			0x0004					// SM hard reset.
#define RST_APLL			0x0002					// SM audio PLL reset.
#define RST_ENC_DEC			0x0001					// Hard reset to encoder and decoder chips.

// MISC register masks.
#define MISC_SM_BSWAP_ON	0x0008					// Swap bytes between GA and SM for MPEG data transfers.
#define MISC_SM_BSWAP_OFF	0x0000					// Pass-through bytes between GA and SM for params upload.

// Flags for set_stream_bypass() calls.
#define BYPASS_ON			1						// Enable encoder-decoder loopback through gate array (bypass SM).
#define BYPASS_OFF			0						// Disable loopback, force data through the SM.




/*******************************************************************************/
// Begin DMA capture specific definitions.
//Upload constants
#define UPLD_D1_B               0x04000400
#define UPLD_D1_A               0x02000200
#define UPLD_BRS                0x01000100
#define UPLD_HPS_H              0x00400040
#define UPLD_HPS_V              0x00200020
#define UPLD_DMA3               0x00100010
#define UPLD_DMA2               0x00080008
#define UPLD_DMA1               0x00040004
#define UPLD_ALLVIDEO           0x07640764
#define UPLD_DEBI               0x00020002
#define UPLD_IIC                0x00010001
#define UPLD_TR_E_1             0x00400040
#define UPLD_TR_D_1             0x00400000

#define UPLD_TR_E_2             0x00200020
#define UPLD_TR_D_2             0x00200000
#define UPLD_TR_E_D             0x00800080
#define UPLD_TR_D_D             0x00800000

//DEBI address stubs
#define STATUS_PSR_GPIO3    0x40        //GPIO3 status bit in PSR
#define STATUS_MC2_SIG0     0x800       //SIG0 status in MC2
#define STATUS_MC2_SIG1     0x1000      //SIG1 status in MC2
#define STATUS_MC2_SIG2     0x2000      //SIG2 status in MC2

#define ISR_DEBI_S          0x00080000  //DEBI status int stat
#define ISR_PIN3            0x00000040  //GPIO3 int stat
#define PSR_PIN3            0x00000040  //GPIO3 PSR status
#define PSR_PIN2            0x00000020  //GPIO2 PSR status
#define PSR_PIN1            0x00000010  //GPIO1 PSR status

//Crosspoint constants
#define XP_PREINIT          0xaaaa9999  //xp pre-initialization
#define XP_INIT             0xaaaa0123  //initial xp setting: inp's 1-4 connected
#define CHAN_ON             0x9         //channel initialization
#define CHAN_OFF            0xa         //channel off


//DEBI constants
//7146 initialization constants
//#define D1_INIT         0x00000400        //D1 ports init value
#define D1_INIT         (0x5<<24 | 0x4<<8) // | 0x2<<22)   //field 2 ends   0x05000400        //D1 ports init value
#define D1_INIT_CAPTURE_FULL (0x1<<24) // | (0x0<<8) | (0x1 << 6)) // | 0x3<<22)
#define D1_INIT_CAPTURE_HALF (0x5<<24) // | (0x0<<8) | (0x1 << 6)) // | 0x3<<22)
#define BURST_INIT      0x1c000017        //DEBI:128 dw burst; video ch1: 32 dw burst, 32 dw threshold
//#define HPS_INIT        (0x50000000 | 4<<12)        //HPS:inp from portB,vert.offs.
#define HPS_INIT        (0x00000000 | 4<<12)        //HPS:inp from portA,vert.offs.

//#define HPSHPS_INIT_LRG     (1<<17 | 1<<18 | 1<<27) //HPS,hor.prescaler
#define HPSHPS_INIT_LRG     (0<<17 | 0<<18 | 1<<27) //HPS,hor.prescaler

#define HPSHFS_INIT_LRG     (1408<<12 | 0x0)       //HPS,hor.fine scaler 192
//#define HPSHFS_INIT_LRG     (2253<<12 | 0x0)       //HPS,hor.fine scaler 120

#define HPSVS_INIT_LRG      (0<<31 | 270<<21 | 0<<15 | 0x26<<8 | 0x26<<1) //HPS,vert.scale 256
//#define HPSVS_INIT_LRG      (0<<31 | 1000<<21 | 0<<15 | 0x26<<8 | 0x26<<1) //HPS,vert.scale 160


//#define HPSVSG_INIT_LRG     (1<<16 | 0x0102)        //HPS,vert.scale gain
#define HPSVSG_INIT_LRG     (1<<16 | 0x101)        //HPS,vert.scale gain
#define HPSHPS_INIT_SML     (0<<17 | 4<<18 | 3<<27 | 0x0404)  //HPS,hor.prescaler
#define HPSHFS_INIT_SML     (0x466<<12 | 0xe)       //HPS,hor.fine scaler

#define HPSVS_INIT_SML      (1<<31 | 0x333<<21 | 4<<15 | 0x33<<8 | 0x33<<1) //HPS,vert.scale
#define HPSVSG_INIT_SML     (2<<16 | 0x0304)        //HPS,vert.scale gain

#define SCALER_INP_LINES_PIX  (504 << 16 | 704)     //active lines/pixels @ 7146 scaler input

#define BCS_INIT            0x80400040        //Bright.,Contrast,Saturation
#define HPSFORM_INIT_MONO   0x06000000        //monochrome
#define HPSFORM_INIT_YCRCB  0x00000000        //YCrCb
#define HPSFORM_INIT_RGB    (2<<30 | 1<<24)   //RGB
#define VDP_DEBI_ENABLE     0x0c000c00        //enable video port and DEBI pins in MC1;

#define EN_RPS0             0x10001000        // enable RPS0

// only for overlay
#define EN_DMA1_RPS0_OVL        0x10401040        //enable RPS0 and video1
//#define EN_DMA2_RPS0        0x10201020        //enable RPS0 and video2
#define EN_RPS1             0x20002000        //enable RPS1
#define DSBL_RPS            0x30400000        //disable RPS1,2, video1
#define DSBL_RPS1           0x20000000        //disable RPS1
#define DSBL_DEBI           0x00800000        //disable DEBI xfer

//RPS command codes capture specific
#define RPS_CLRSIGNAL           0x00000000      //CLEAR SIGNAL
#define RPS_NOP                 0x00000000      //NOP
#define RPS_SETSIGNAL           0x10000000      //SET SIGNAL
#define RPS_PAUSE               0x20000000      //PAUSE
#define RPS_CHKLATE             0x30000000      //CHECK-LATE
#define RPS_UPLOAD              0x40000000      //UPLOAD
#define RPS_STOP                0x50000000      //STOP
#define RPS_IRQ                 0x60000000      //INTERRUPT
#define RPS_JUMP                0x80000000      //JUMP
#define RPS_LDREG               0x90000000      //LDREG
#define RPS_STREG               0xa0000000      //STREG
#define RPS_MSKLOAD             0xc0000000      //MASKLOAD
#define RPS_OR                  0x08000000      //OR conditions
#define RPS_INV                 0x04000000      //invert conditions
#define RPS_GPIO3               0x00100000      //GPIO3
#define RPS_HS                  0x00008000      //HS done
#define RPS_HT                  0x00010000      //HT done
#define RPS_VBIA                0x00000400      //VBI_A
#define RPS_VBIB                0x00000200      //VBI_B
#define RPS_HPS                 0x00000020      //HPS done
#define RPS_VTD1                0x00000004      //VTD1
#define RPS_DEBI                0x00000002      //DEBI done
#define RPS_SIG0                0x00200000      //RPS signal 0 (in RPS command)
#define RPS_STAT_SIG0           0x00000800      //RPS signal 0 (in MC2)
#define RPS_SIG1                0x00400000      //RPS signal 1 (in RPS command)
#define RPS_STAT_SIG1           0x00001000      //RPS signal 1 (in MC2)
#define RPS_SIG2                0x00800000      //RPS signal 2 (in RPS command)
#define RPS_STAT_SIG2           0x00002000      //RPS signal 2 (in MC2)
#define RPS_SIG3                0x01000000      //RPS signal 3 (in RPS command)
#define RPS_SIG4                0x02000000      //RPS signal 4 (in RPS command)
#define RPS_IRQI0               0x08000000      //RPS interrupt I0;
#define RPS_LINE                0x00000040      //RPS line;
#define RPS_E_FIDA              0x00000800      //RPS even field 2 portA;
#define RPS_E_FIDB              0x00002000      //RPS even field 2 portB;
#define RPS_O_FIDA              0x00001000      //RPS odd field 2 portA;
#define RPS_O_FIDB              0x00004000      //RPS odd field 2 portB;
#define RPS_DEBI_UPLD           0x6             //DEBI upload const

//Video DMA control constants
#define DMA_MAP                 0x800           //DMA mapping enable
#define DMA_PROTVIOL            0x8             //DMA protection violation handling
#define DMA_SWAP2               1               //2 byte swap
#define DMA_NOSWAP              0               //no swap



#endif  //SX16_KERNEL

#endif		// #ifndef DEF_CORE_H

