/* 616 demo.
 * Copyright 2013 Sensoray Co. Inc.
 * Writing to file and modifying the stream
 * to comply with QBA (quad byte alignment) requirement
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <sched.h>
#include <time.h>
#include <sys/ioctl.h>
#include <assert.h>

#define MPEG_BUF_SIZE	4096
unsigned char mpeg_buf[MPEG_BUF_SIZE*2];

/* Board handles*/
// max bitrate is 20Mbps == 2.5 Mbytes/s
#define MAX_RBUF_SIZE (1024*100)  // will be much smaller
#define QBA_STATE_FAIL 5

struct qba_state {
    int cur_pes; // current PES of interest
    int pes_len;
    int last_slice; // position of last slice
    int picture; // position of picture
    int search; // internal search marker
    int rbuf_pos; // buffer position
    int state;
};

static struct qba_state qba[1];

// returns position if found, otherwise 0
// state updated in qba
// searches for code starting at start
// len is size of buffer.
// code is code to search for
int find_start_code(unsigned char *buf, int start, int len, int code, struct qba_state *qba)
{
    int i = 0;
    int find_any = (code > 0xff) ? 1 : 0;
    for (i = start; i < len; i++) {
        switch (qba->search) {
        case 0:
        case 1:
            // search for 0x00
            if (buf[i] == 0x00) {
                qba->search++;
//                printf("[%d]found 0\n", i);
            } else {
                qba->search = 0;
            }
            break;
        case 2:
            // search for 0x01
            if (buf[i] == 0x01) {
                //printf("[%d]found 1\n", i);
                qba->search++;
            } else if (buf[i] != 0x00) {
                //printf("fail %x %x %x %x\n", buf[i-3], buf[i-2], buf[i-1], buf[i]);
                qba->search = 0;
            }
            break;
        case 3:
            if ((buf[i] == code) || find_any) {
                qba->search = 0;
                //printf("[%d]found 0x%x\n", i, buf[i]);
                return i-3;
            } else
                qba->search = 0;
            break;
        default:
            qba->search = 0;
            break;
        }
    }
    return 0;
}

// reverse find start code
int find_start_code_rev(unsigned char *buf, int start, int code)
{
    int i = 0;
    int state = 0;
    int find_any = (code > 0xff) ? 1 : 0;
    for (i = start-3; i >= 0; i--) {
        if ((buf[i] == 0x00) && (buf[i+1] == 0x00) && (buf[i+2] == 0x01) && (buf[i+3] == code))
            return i;
    }
    return -1;
}

void dump_bytes(unsigned char *buf, int bytes)
{
    int i;
    for (i = 0; i < bytes; i++) {
        printf(" 0x%02x", buf[i]);
        if (i % 8 == 7) {
            printf("\n");
        }
    }
    return;
}

///////////////////////////////////////////////////////////////////////////
// Init the system and execute user commands until termination.

int main( int argc, char *argv[] )
{
  int h; // handle to board
  char dev_name[100];
  FILE *file, *fdump;
  int tlen = 0;
  int board = 0;
  int rc = 1;
  int added = 0;
  int relpos = 0;
  unsigned char *buf;
  int i = 0;
  int aligned = 0;
  int nonaligned = 0;
  strcpy(dev_name, argv[1]);
#define MAX_FILESIZE (32*1024*1024)
  // open board
  h = open( dev_name, O_RDWR);
  if (h == -1) {
    printf( "Driver open error\n" );
    return( -1 );
  }
  buf = (unsigned char *) malloc(MAX_FILESIZE);
  if (buf == NULL)
      printf("out of memory\n");
  read(h, buf, 32*1024*1024);
  printf("start\n");
  while (rc > 0) {
      rc = find_start_code(buf, relpos, MAX_FILESIZE, 0x1e, &qba[board]);
      if (rc != 0) {
          int rc2;
          relpos = rc+4;
          rc2 = find_start_code(buf, relpos, MAX_FILESIZE, 0xff+1, &qba[board]);
          if ((rc2-rc) % 4)
              nonaligned++;
          else
              aligned++;

          printf("[position: %x] slice length: %03d, QBA: %s, next start_code %x\n", rc, rc2-rc, 
                 (rc2-rc) % 4 ? "NO" : "YES", buf[rc2+3]);

          relpos = rc2;
          i++;
      }
  }
      printf("===========================================\n");
  printf("Number of QBA aligned last_slices %d, Number of non-aligned slices %d\n", aligned, nonaligned);
  if (!nonaligned)
      printf("Congratulations, the file is QBA compliant!\n");
  else
      printf("The file is not QBA compliant\n");

  printf("===========================================\n");

  free(buf);
  close(h);

  return 0;
}

